@@ -2,9 +2,9 @@
 
 Implements the Distutils 'sdist' command (create a source distribution)."""
 
-# created 1999/09/22, Greg Ward
+# This module should be kept compatible with Python 1.5.2.
 
-__revision__ = "$Id: sdist.py,v 1.42 2000/08/05 01:31:54 gward Exp $"
+__revision__ = "$Id: sdist.py,v 1.56.14.2 2004/08/16 12:15:00 doko Exp $"
 
 import sys, os, string
 from types import *
@@ -14,6 +14,7 @@ from distutils import dir_util, dep_util, file_util, archive_util
 from distutils.text_file import TextFile
 from distutils.errors import *
 from distutils.filelist import FileList
+from distutils import log
 
 
 def show_formats ():
@@ -31,7 +32,6 @@ def show_formats ():
     pretty_printer.print_help(
         "List of available source distribution formats:")
 
-
 class sdist (Command):
 
     description = "create a source distribution (tarball, zip file, etc.)"
@@ -59,7 +59,7 @@ class sdist (Command):
          "forcibly regenerate the manifest and carry on as usual"),
         ('formats=', None,
          "formats for source distribution (comma-separated list)"),
-        ('keep-tree', 'k',
+        ('keep-temp', 'k',
          "keep the distribution tree around after creating " +
          "archive file(s)"),
         ('dist-dir=', 'd',
@@ -67,11 +67,14 @@ class sdist (Command):
          "[default: dist]"),
         ]
 
+    boolean_options = ['use-defaults', 'prune',
+                       'manifest-only', 'force-manifest',
+                       'keep-temp']
 
     help_options = [
         ('help-formats', None,
          "list available distribution formats", show_formats),
-	]
+        ]
 
     negative_opt = {'no-defaults': 'use-defaults',
                     'no-prune': 'prune' }
@@ -94,7 +97,7 @@ class sdist (Command):
         self.force_manifest = 0
 
         self.formats = None
-        self.keep_tree = 0
+        self.keep_temp = 0
         self.dist_dir = None
 
         self.archive_files = None
@@ -115,7 +118,7 @@ class sdist (Command):
                       "don't know how to create source distributions " + \
                       "on platform %s" % os.name
 
-        bad_format = archive_util.check_archive_formats (self.formats)
+        bad_format = archive_util.check_archive_formats(self.formats)
         if bad_format:
             raise DistutilsOptionError, \
                   "unknown archive format '%s'" % bad_format
@@ -129,15 +132,15 @@ class sdist (Command):
         # 'filelist' contains the list of files that will make up the
         # manifest
         self.filelist = FileList()
-        
+
         # Ensure that all required meta-data is given; warn if not (but
         # don't die, it's not *that* serious!)
-        self.check_metadata ()
+        self.check_metadata()
 
         # Do whatever it takes to get the list of files to process
         # (process the manifest template, read an existing manifest,
         # whatever).  File list is accumulated in 'self.filelist'.
-        self.get_file_list ()
+        self.get_file_list()
 
         # If user just wanted us to regenerate the manifest, stop now.
         if self.manifest_only:
@@ -145,7 +148,7 @@ class sdist (Command):
 
         # Otherwise, go ahead and create the source distribution tarball,
         # or zipfile, or whatever.
-        self.make_distribution ()
+        self.make_distribution()
 
 
     def check_metadata (self):
@@ -158,25 +161,25 @@ class sdist (Command):
 
         missing = []
         for attr in ('name', 'version', 'url'):
-            if not (hasattr (metadata, attr) and getattr (metadata, attr)):
-                missing.append (attr)
+            if not (hasattr(metadata, attr) and getattr(metadata, attr)):
+                missing.append(attr)
 
         if missing:
-            self.warn ("missing required meta-data: " +
-                       string.join (missing, ", "))
+            self.warn("missing required meta-data: " +
+                      string.join(missing, ", "))
 
         if metadata.author:
             if not metadata.author_email:
-                self.warn ("missing meta-data: if 'author' supplied, " +
-                           "'author_email' must be supplied too")
+                self.warn("missing meta-data: if 'author' supplied, " +
+                          "'author_email' must be supplied too")
         elif metadata.maintainer:
             if not metadata.maintainer_email:
-                self.warn ("missing meta-data: if 'maintainer' supplied, " +
-                           "'maintainer_email' must be supplied too")
+                self.warn("missing meta-data: if 'maintainer' supplied, " +
+                          "'maintainer_email' must be supplied too")
         else:
-            self.warn ("missing meta-data: either (author and author_email) " +
-                       "or (maintainer and maintainer_email) " +
-                       "must be supplied")
+            self.warn("missing meta-data: either (author and author_email) " +
+                      "or (maintainer and maintainer_email) " +
+                      "must be supplied")
 
     # check_metadata ()
 
@@ -202,7 +205,10 @@ class sdist (Command):
         # manifest, but there's no template -- which will happen if the
         # developer elects to generate a manifest some other way -- then we
         # can't regenerate the manifest, so we don't.)
-        setup_newer = dep_util.newer(sys.argv[0], self.manifest)
+        self.debug_print("checking if %s newer than %s" %
+                         (self.distribution.script_name, self.manifest))
+        setup_newer = dep_util.newer(self.distribution.script_name,
+                                     self.manifest)
 
         # cases:
         #   1) no manifest, template exists: generate manifest
@@ -216,39 +222,29 @@ class sdist (Command):
         #      do nothing (unless --force or --manifest-only)
         #   4) no manifest, no template: generate w/ warning ("defaults only")
 
-        # Regenerate the manifest if necessary (or if explicitly told to)
-        if ((template_exists and (template_newer or setup_newer)) or
-            self.force_manifest or self.manifest_only):
+        manifest_outofdate = (template_exists and
+                              (template_newer or setup_newer))
+        force_regen = self.force_manifest or self.manifest_only
+        manifest_exists = os.path.isfile(self.manifest)
+        neither_exists = (not template_exists and not manifest_exists)
 
+        # Regenerate the manifest if necessary (or if explicitly told to)
+        if manifest_outofdate or neither_exists or force_regen:
             if not template_exists:
                 self.warn(("manifest template '%s' does not exist " +
                            "(using default file list)") %
                           self.template)
-
             self.filelist.findall()
 
-            # Add default file set to 'files'
             if self.use_defaults:
                 self.add_defaults()
-
-            # Read manifest template if it exists
             if template_exists:
                 self.read_template()
-
-            # Prune away any directories that don't belong in the source
-            # distribution
             if self.prune:
                 self.prune_file_list()
 
-            # File list now complete -- sort it so that higher-level files
-            # come first
             self.filelist.sort()
-
-            # Remove duplicates from the file list
             self.filelist.remove_duplicates()
-
-            # And write complete file list (including default file set) to
-            # the manifest.
             self.write_manifest()
 
         # Don't regenerate the manifest, just read it in.
@@ -270,65 +266,65 @@ class sdist (Command):
         else is optional.
         """
 
-        # XXX name of setup script and config file should be taken
-        # programmatically from the Distribution object (except
-        # it doesn't have that capability... yet!)
-        standards = [('README', 'README.txt'), 'setup.py']
+        standards = [('README', 'README.txt'), self.distribution.script_name]
         for fn in standards:
-            if type (fn) is TupleType:
+            if type(fn) is TupleType:
                 alts = fn
                 got_it = 0
                 for fn in alts:
-                    if os.path.exists (fn):
+                    if os.path.exists(fn):
                         got_it = 1
-                        self.filelist.append (fn)
+                        self.filelist.append(fn)
                         break
 
                 if not got_it:
-                    self.warn ("standard file not found: should have one of " +
-                               string.join (alts, ', '))
+                    self.warn("standard file not found: should have one of " +
+                              string.join(alts, ', '))
             else:
-                if os.path.exists (fn):
-                    self.filelist.append (fn)
+                if os.path.exists(fn):
+                    self.filelist.append(fn)
                 else:
-                    self.warn ("standard file '%s' not found" % fn)
+                    self.warn("standard file '%s' not found" % fn)
 
         optional = ['test/test*.py', 'setup.cfg']
         for pattern in optional:
-            files = filter (os.path.isfile, glob (pattern))
+            files = filter(os.path.isfile, glob(pattern))
             if files:
-                self.filelist.extend (files)
+                self.filelist.extend(files)
 
         if self.distribution.has_pure_modules():
-            build_py = self.get_finalized_command ('build_py')
-            self.filelist.extend (build_py.get_source_files ())
+            build_py = self.get_finalized_command('build_py')
+            self.filelist.extend(build_py.get_source_files())
 
         if self.distribution.has_ext_modules():
-            build_ext = self.get_finalized_command ('build_ext')
-            self.filelist.extend (build_ext.get_source_files ())
+            build_ext = self.get_finalized_command('build_ext')
+            self.filelist.extend(build_ext.get_source_files())
 
         if self.distribution.has_c_libraries():
-            build_clib = self.get_finalized_command ('build_clib')
-            self.filelist.extend (build_clib.get_source_files ())
+            build_clib = self.get_finalized_command('build_clib')
+            self.filelist.extend(build_clib.get_source_files())
+
+        if self.distribution.has_scripts():
+            build_scripts = self.get_finalized_command('build_scripts')
+            self.filelist.extend(build_scripts.get_source_files())
 
     # add_defaults ()
-    
+
 
     def read_template (self):
+        """Read and parse manifest template file named by self.template.
 
-        """Read and parse the manifest template file named by
-        'self.template' (usually "MANIFEST.in").  The parsing and
-        processing is done by 'self.filelist', which updates itself
-        accordingly.
+        (usually "MANIFEST.in") The parsing and processing is done by
+        'self.filelist', which updates itself accordingly.
         """
-        self.announce("reading manifest template '%s'" % self.template)
-        template = TextFile (self.template,
-                             strip_comments=1,
-                             skip_blanks=1,
-                             join_lines=1,
-                             lstrip_ws=1,
-                             rstrip_ws=1,
-                             collapse_ws=1)
+        log.info("reading manifest template '%s'", self.template)
+        template = TextFile(self.template,
+                            strip_comments=1,
+                            skip_blanks=1,
+                            join_lines=1,
+                            lstrip_ws=1,
+                            rstrip_ws=1,
+                            collapse_join=1)
 
         while 1:
             line = template.readline()
@@ -350,7 +346,7 @@ class sdist (Command):
         by 'read_template()', but really don't belong there:
           * the build tree (typically "build")
           * the release tree itself (only an issue if we ran "sdist"
-            previously with --keep-tree, or it aborted)
+            previously with --keep-temp, or it aborted)
           * any RCS or CVS directories
         """
         build = self.get_finalized_command('build')
@@ -358,7 +354,7 @@ class sdist (Command):
 
         self.filelist.exclude_pattern(None, prefix=build.build_base)
         self.filelist.exclude_pattern(None, prefix=base_dir)
-        self.filelist.exclude_pattern(r'/(RCS|CVS)/.*', is_regex=1)
+        self.filelist.exclude_pattern(r'/(RCS|CVS|\.svn)/.*', is_regex=1)
 
 
     def write_manifest (self):
@@ -378,18 +374,18 @@ class sdist (Command):
         fill in 'self.filelist', the list of files to include in the source
         distribution.
         """
-        self.announce("reading manifest file '%s'" % self.manifest)
-        manifest = open (self.manifest)
+        log.info("reading manifest file '%s'", self.manifest)
+        manifest = open(self.manifest)
         while 1:
-            line = manifest.readline ()
+            line = manifest.readline()
             if line == '':              # end of file
                 break
             if line[-1] == '\n':
                 line = line[0:-1]
-            self.filelist.append (line)
+            self.filelist.append(line)
 
     # read_manifest ()
-            
+
 
     def make_release_tree (self, base_dir, files):
         """Create the directory tree that will become the source
@@ -401,9 +397,10 @@ class sdist (Command):
         to be distributed.
         """
         # Create all the directories under 'base_dir' necessary to
-        # put 'files' there.
-        dir_util.create_tree (base_dir, files,
-                              verbose=self.verbose, dry_run=self.dry_run)
+        # put 'files' there; the 'mkpath()' is just so we don't die
+        # if the manifest happens to be empty.
+        self.mkpath(base_dir)
+        dir_util.create_tree(base_dir, files, dry_run=self.dry_run)
 
         # And walk over the list of files, either making a hard link (if
         # os.link exists) to each one that doesn't already exist in its
@@ -411,28 +408,35 @@ class sdist (Command):
         # that's out-of-date in 'base_dir'.  (Usually, all files will be
         # out-of-date, because by default we blow away 'base_dir' when
         # we're done making the distribution archives.)
-    
-        if hasattr (os, 'link'):        # can make hard links on this system
+
+        if hasattr(os, 'link'):        # can make hard links on this system
             link = 'hard'
             msg = "making hard links in %s..." % base_dir
         else:                           # nope, have to copy
             link = None
             msg = "copying files to %s..." % base_dir
 
-        self.announce (msg)
+        if not files:
+            log.warn("no files to distribute -- empty manifest?")
+        else:
+            log.info(msg)
         for file in files:
-            dest = os.path.join (base_dir, file)
-            self.copy_file (file, dest, link=link)
+            if not os.path.isfile(file):
+                log.warn("'%s' not a regular file -- skipping" % file)
+            else:
+                dest = os.path.join(base_dir, file)
+                self.copy_file(file, dest, link=link)
+
+        self.distribution.metadata.write_pkg_info(base_dir)
 
     # make_release_tree ()
 
-
     def make_distribution (self):
         """Create the source distribution(s).  First, we create the release
         tree with 'make_release_tree()'; then, we create all required
         archive files (according to 'self.formats') from the release tree.
         Finally, we clean up by blowing away the release tree (unless
-        'self.keep_tree' is true).  The list of archive files created is
+        'self.keep_temp' is true).  The list of archive files created is
         stored so it can be retrieved later by 'get_archive_files()'.
         """
         # Don't warn about missing meta-data here -- should be (and is!)
@@ -440,18 +444,16 @@ class sdist (Command):
         base_dir = self.distribution.get_fullname()
         base_name = os.path.join(self.dist_dir, base_dir)
 
-        self.make_release_tree (base_dir, self.filelist.files)
+        self.make_release_tree(base_dir, self.filelist.files)
         archive_files = []              # remember names of files we create
-        if self.dist_dir:
-            self.mkpath(self.dist_dir)
         for fmt in self.formats:
-            file = self.make_archive (base_name, fmt, base_dir=base_dir)
+            file = self.make_archive(base_name, fmt, base_dir=base_dir)
             archive_files.append(file)
 
         self.archive_files = archive_files
 
-        if not self.keep_tree:
-            dir_util.remove_tree (base_dir, self.verbose, self.dry_run)
+        if not self.keep_temp:
+            dir_util.remove_tree(base_dir, dry_run=self.dry_run)
 
     def get_archive_files (self):
         """Return the list of archive files created when the command
